// Copyright (c) 2019-2020 The Authors
// This file is part of @npcz/ngx-search-bar
// https://github.com/abdes/happy-coding
//
// SPDX-License-Identifier: BSD-3-Clause
import { Component, Input, Output, EventEmitter, ViewChild, ElementRef, } from '@angular/core';
import { FormBuilder } from '@angular/forms';
import { MatButton } from '@angular/material/button';
import { MatAutocomplete } from '@angular/material/autocomplete';
import { Subject } from 'rxjs';
import { takeUntil } from 'rxjs/operators';
export class SearchBarComponent {
    constructor(_fb) {
        this._fb = _fb;
        this.highlightStyle = {
            hover: 'background',
            focus: 'outline',
        };
        this.search = new EventEmitter();
        this.changes = new EventEmitter();
        this.searchQuery = '';
        // Public because of use in the template, but not intended to be used outside
        // of this component
        this._searchForm = this._fb.group({
            searchInput: null,
        });
        // Automatic unsubscription when the component is destroyed
        // We don't want an emitted value, just the fact of emitting is enough
        this._ngUnsubscribe$ = new Subject();
    }
    get categoryList() {
        return this._categoryList;
    }
    set categoryList(value) {
        if (value && value.length > 0)
            this._categoryList = value;
        else
            this._categoryList = null;
        if (!this.category && this._categoryList) {
            this.category = this._categoryList[0];
        }
    }
    get autoComplete() {
        return this._autoComplete;
    }
    set autoComplete(value) {
        this._autoComplete = value;
        this._setupAutoCompleteSelectionHandler();
    }
    ngOnInit() {
        if (this.disabled)
            this._searchForm.get('searchInput').disable();
        this._searchForm
            .get('searchInput')
            .valueChanges.pipe(takeUntil(this._ngUnsubscribe$))
            .subscribe((value) => {
            if (this.searchQuery != value) {
                this.searchQuery = value;
                //console.debug(`emit changes: ${this.category} / ${this.searchQuery}`);
                this.changes.emit({
                    category: this.category,
                    query: this.searchQuery,
                });
            }
        });
    }
    ngOnDestroy() {
        this._ngUnsubscribe$.next();
        this._ngUnsubscribe$.complete();
    }
    clear() {
        this._searchForm.get('searchInput').setValue('');
        this._setFocusToSearchInput();
    }
    enable() {
        this._searchForm.enable();
        this.disabled = false;
    }
    disable() {
        this._searchForm.disable();
        this.disabled = true;
    }
    focus() {
        this._setFocusToSearchInput();
    }
    _onCategorySelected(value) {
        if (this.category != value) {
            this.category = value;
            // console.debug(`emit changes: ${this.category} / ${this.searchQuery}`);
            this.changes.emit({
                category: this.category,
                query: this.searchQuery,
            });
        }
    }
    _onCategoryMenuClosed() {
        this._setFocusToSearchInput();
    }
    _onSubmit() {
        // console.debug(`submit: ${this.category} / ${this.searchQuery}`);
        this.search.emit({
            category: this.category,
            query: this.searchQuery,
        });
        this._searchInput.nativeElement.blur();
    }
    _setupAutoCompleteSelectionHandler() {
        this._autoComplete.optionSelected
            .pipe(takeUntil(this._ngUnsubscribe$))
            .subscribe(() => {
            // Trigger a form submit by clicking the submit button
            this._submitButton._elementRef.nativeElement.click();
        });
    }
    _setFocusToSearchInput() {
        this._searchInput.nativeElement.focus();
        // Move the cursor to the end of the text input value so that the user is
        // ready to type
        this._moveCursorToEnd();
    }
    _moveCursorToEnd() {
        const el = this._searchInput.nativeElement;
        el.selectionStart = el.selectionEnd = el.value.length;
    }
}
SearchBarComponent.decorators = [
    { type: Component, args: [{
                selector: 'hc-search-bar',
                template: "<!--\n  ~ Copyright (c) 2019-2020 The Authors\n  ~ This file is part of @npcz/ngx-search-bar\n  ~ https://github.com/abdes/happy-coding\n  ~\n  ~ SPDX-License-Identifier: BSD-3-Clause\n  -->\n\n<form\n  [formGroup]=\"_searchForm\"\n  class=\"search__wrapper\"\n  [hcHighlight]=\"highlightStyle\"\n  (ngSubmit)=\"_onSubmit()\"\n>\n  <div class=\"search__container\" fxLayout=\"row\">\n    <!--\n      Set the type to 'button' so that the only submit button in the form is\n      the 'search__submit' button\n    -->\n    <button\n      type=\"button\"\n      *ngIf=\"categoryList\"\n      mat-stroked-button\n      class=\"search-category__button\"\n      [disabled]=\"disabled\"\n      [matMenuTriggerFor]=\"categoryMenu\"\n      (menuClosed)=\"_onCategoryMenuClosed()\"\n    >\n      <div fxLayout=\"row\" fxFlex=\"grow\">\n        <div\n          class=\"search-category__text\"\n          fxFlex=\"grow\"\n          ellipsis\n          [ellipsis-content]=\"category\"\n        ></div>\n        <mat-icon fxFlexAlign=\"center\">arrow_drop_down</mat-icon>\n      </div>\n    </button>\n\n    <!--\n      Because autocomplete is optional, we avoid errors on the input field by\n      setting the matAutocompleteDisabled property based on whether we do have\n      an autoComplete component set or not.\n    -->\n    <input\n      #search__input\n      type=\"text\"\n      matInput\n      class=\"search__input\"\n      fxFlex=\"grow\"\n      [placeholder]=\"placeholder\"\n      formControlName=\"searchInput\"\n      [matAutocomplete]=\"autoComplete\"\n      [matAutocompleteDisabled]=\"!autoComplete\"\n    />\n    <div *ngIf=\"searchQuery && clearButton\" fxLayout=\"row\">\n      <!--\n        Set the type to 'button' so that the only submit button in the form is\n        the 'search__submit' button\n      -->\n      <button\n        #search__clear\n        class=\"search__clear\"\n        [disabled]=\"disabled\"\n        type=\"button\"\n        mat-icon-button\n        aria-label=\"Clear\"\n        (click)=\"clear()\"\n      >\n        <mat-icon>close</mat-icon>\n      </button>\n      <div fxFlexAlign=\"center\" class=\"clear__separator\"></div>\n    </div>\n    <!-- This is the only button that must be set with type 'submit' -->\n    <button\n      #search__submit\n      class=\"search__submit\"\n      type=\"submit\"\n      mat-icon-button\n      [disabled]=\"disabled\"\n    >\n      <mat-icon>search</mat-icon>\n    </button>\n  </div>\n</form>\n\n<mat-menu #categoryMenu=\"matMenu\">\n  <ng-template matMenuContent>\n    <button\n      *ngFor=\"let category of categoryList\"\n      (click)=\"_onCategorySelected(category)\"\n      mat-menu-item\n    >\n      {{ category }}\n    </button>\n  </ng-template>\n</mat-menu>\n",
                styles: [".clear__separator{border-right:thin;border-right-style:solid;height:60%;margin-right:5px;width:5px}.search__input{border:none;margin:0;outline:none}.search__wrapper{border-radius:8px;margin:4px;padding:5px}.search-category__button{flex:1 0 20%;margin-right:5px;max-width:160px;min-width:80px;padding:0 0 0 5px}.search-category__text{overflow:hidden}"]
            },] }
];
SearchBarComponent.ctorParameters = () => [
    { type: FormBuilder }
];
SearchBarComponent.propDecorators = {
    _searchInput: [{ type: ViewChild, args: ['search__input',] }],
    _submitButton: [{ type: ViewChild, args: ['search__submit',] }],
    placeholder: [{ type: Input }],
    clearButton: [{ type: Input }],
    categoryList: [{ type: Input }],
    category: [{ type: Input }],
    disabled: [{ type: Input }],
    highlightStyle: [{ type: Input }],
    autoComplete: [{ type: Input }],
    search: [{ type: Output }],
    changes: [{ type: Output }]
};
//# sourceMappingURL=search-bar.component.js.map